# 	Copyright (c) 2005-2008 Peter V. Saveliev
#
# 	This file is part of Connexion project.
#
# 	Connexion is free software; you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation; either version 3 of the License, or
# 	(at your option) any later version.
#
# 	Connexion is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with Connexion; if not, write to the Free Software
# 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from command import command
from cxutil import flags,pmask,cmask
from cxutil.exceptions import Dump,BranchPass
from pysnmp.smi import builder, view, error

import re
import types

F_EMPTY	= 0
F_LIST	= 1
F_NOT	= 2

class catch (command):
	'''
	Catch an event
	'''

	mask = flags.Begin | flags.Transparent

	parameters = {
		"$1": {
			"type": "string",
			"description": "Signal ID",
			"name": "sigid",
		},
		"if": {
			"type": "word",
			"description": "Signal field match",
			"mask": pmask.Optional | pmask.List,
			"name": "condition",
		}
	}

	def immediate(self,opts):
		self.re = {
			"dot": re.compile("^([0-9].?)+$"),
			"fqn": re.compile("^[a-zA-Z-]+:+\S+$"),
		}
		self.catch = (opts.envl.sigid,)


	def lookup(self,oid):
		oid = str(oid)

		if self.re["dot"].match(oid):
			result = oid.split(".")
			for i in xrange(len(result)):
				result[i] = int(result[i])
			result = tuple(result)

		else:
			if self.re["fqn"].match(oid):
				result = oid.split(":")
				mibBuilder = builder.MibBuilder().loadModules(result[0])
				target = result[-1]
			else:
				mibBuilder = builder.MibBuilder().loadModules('SNMPv2-MIB','IF-MIB')
				target = oid

			mibView = view.MibViewController(mibBuilder)
			try:
				result, label, suffix = mibView.getNodeName((target,))
			except:
				result = oid

		return result
		

	def pre(self,opts):
		if \
			(not opts.call_mask & cmask.Signal) and\
			(not opts.call_mask & cmask.Immediate):
			raise BranchPass

		result = command.pre(self,opts)

		if opts.envl.sigid == "*":
			return

		# analyze signal
		if opts.call_mask & cmask.Signal:
			s = opts.envl.signal

			for i in opts.envl.condition:

				flags = F_EMPTY

				if i[0] == "!":
					flags |= F_NOT
					i = i[1:]

				a = i.split("=")
				if len(a) > 1:
					b = "=".join(a[1:])
				else:
					a = i.split("&")
					if len(a) > 1:
						b = "&".join(a[1:])
						flags |= F_LIST
					else:
						b = None
				a = a[0]

				###
				# Protocol-specific: SNMP // FIXME: how to plug-in such code?
				###

				if opts.envl.sigid.startswith("snmp"):
					a = self.lookup(a)
					b = self.lookup(b)
					if type(a) == types.TupleType:
						aX = str(a)[1:-1]
					else:
						aX = str(a)

					try:
						for (i,k) in s.items():
							if type(i) == types.TupleType:
								iX = str(i)[1:-1]
								if iX.startswith(aX):
									if (b == k[-1]) or (b is None):
										raise
							else:
								iX = str(i)
								if (iX == aX):
									if (b == k) or (b is None):
										raise
					except:
						continue

					raise BranchPass

				###
				# Generic protocol
				###
				else:
					if s.has_key(a):
						if b is None:
							dod(flags,True)
							continue

						if flags & F_LIST:
							dod(flags,b in s[a])
							continue
						else:
							dod(flags,s[a] == b)
							continue
					else:
						raise BranchPass

		return

def dod(flags,condition):
	if flags & F_NOT:
		if condition:
			raise BranchPass
	else:
		if not condition:
			raise BranchPass
