"""
An event provider: DNS queries
"""

# 	Copyright (c) 2008 Peter V. Saveliev
#
# 	This file is part of Connexion project.
#
# 	Connexion is free software; you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation; either version 3 of the License, or
# 	(at your option) any later version.
#
# 	Connexion is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with Connexion; if not, write to the Free Software
# 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from dns import message
from cxutil import flags,cmask
from socket import socket,AF_INET,SOCK_DGRAM
from command import command

class dns_socket(object):
	"""
	DNS server socket
	"""
	fd = None
	sock = None
	signal = None

	def __init__(self,signal,bind,port):
		self.signal = signal
		self.sock = socket(AF_INET,SOCK_DGRAM)
		self.sock.bind((bind,port))
		self.fd = self.sock.fileno()

	def read(self):
		s = self.sock.recvfrom(65535)
		b = {}
		
		b["src"] = s[1][0]
		b["sport"] = s[1][1]
		b["query"] = message.from_wire(s[0])

		self.signal("dns.query",b)

class dnsq (command):
	"""
	An event provider: DNS queries
	"""
	mask = flags.Esoteric

	parameters = {
		"$1": {
			"type": "word",
			"name": "bind",
			"description": "An address to bind to (default: all interfaces)",
			"default": "",
		},
		"port": {
			"type": "int",
			"description": "A port to bind to (default: 53) (currently only UDP is supported)",
			"default": 53,
		},
	}

	def init(self,opts):
		if not opts.call_mask & cmask.Immediate:
			self.s = dns_socket(self.signal,opts.envl.bind,int(opts.envl.port))
			opts.esoteric.bus.put("monitor",("register",self.s.fd,self.s.read))

	def postUp(self,opts):
		pass
